﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Xml;
using railML.rail;

namespace TrainPartAssignmentCheck
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void btnSelectFileRailML_Click(object sender, EventArgs e)
        {
            if (openRailMLDlg.ShowDialog() == DialogResult.OK)
            {
                txtFileRailML.Text = openRailMLDlg.FileName;
            }
        }

        private void btnCheck_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(txtFileRailML.Text))
            {
                MessageBox.Show("Need to specify the railML 2.2 file to check.");
                return;
            }
            List<string> trainPartIds = new List<string>();
            var document = railML2.LoadFromFile(txtFileRailML.Text);
            txtNumberOfTPs.Text = document.railml3.First.timetable.First.trainParts.First.trainPart.Count.ToString();
            foreach (eTrainPart trainPart in document.railml3.First.timetable.First.trainParts.First.trainPart)
            {
                trainPartIds.Add(trainPart.id.Value);
            }

            foreach (eTrain train in document.railml3.First.timetable.First.trains.First.train)
            {
                foreach (eTrainPartSequence trainPartSequence in train.trainPartSequence)
                {
                    foreach (tTrainPartRef trainPartRef in trainPartSequence.trainPartRef)
                    {
                        trainPartIds.Remove(trainPartRef.ref2.Value);
                    }
                }
            }

            txtAssigned.Text =
                (document.railml3.First.timetable.First.trainParts.First.trainPart.Count - trainPartIds.Count)
                .ToString();
            txtUnassigned.Text = trainPartIds.Count.ToString();
            lstUnassignedTPs.Items.Clear();
            trainPartIds.ForEach(x => lstUnassignedTPs.Items.Add(x));
            int blockPartProblemsDueToMissingTrainPart = 0;
            int blockPartProblemsDueToInconsistentStartAndEnd = 0;
            foreach (tBlockPart blockPart in document.railml3.First.timetable.First.rosterings.First.rostering.First.blockParts.First.blockPart)
            {
                if (blockPart.mission.Exists() && blockPart.mission.Value.Equals("timetable"))
                {
                    if (!blockPart.trainPartRef.Exists())
                    {
                        blockPartProblemsDueToMissingTrainPart++;
                        continue;
                    }

                    eTrainPart trainPart = null;
                    foreach (eTrainPart part in document.railml3.First.timetable.First.trainParts.First.trainPart)
                    {
                        if(part.id.Value.Equals(blockPart.trainPartRef.Value))
                        {
                            trainPart = part;
                            break;
                        }   
                    }

                    if (trainPart == null)
                    {
                        blockPartProblemsDueToMissingTrainPart++;
                        continue;
                    }

                    if (blockPart.startOcpRef.Exists() &&
                        !trainPart.ocpsTT.First.ocpTT.First.ocpRef.Value.Equals(blockPart.startOcpRef.Value))
                    {
                        blockPartProblemsDueToInconsistentStartAndEnd++;
                        continue;
                    }

                    if (blockPart.endOcpRef.Exists() &&
                        !trainPart.ocpsTT.First.ocpTT.Last.ocpRef.Value.Equals(blockPart.endOcpRef.Value))
                    {
                        blockPartProblemsDueToInconsistentStartAndEnd++;
                        continue;
                    }
                }
            }

            txtBlockPartProblems.Text =
                $"{blockPartProblemsDueToMissingTrainPart} / {blockPartProblemsDueToInconsistentStartAndEnd}";

        }

        private void btnCount_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(txtFileRailML.Text))
            {
                MessageBox.Show("Need to specify the railML 2.2 file to check.");
                return;
            }

            Dictionary<string, int> elements = new Dictionary<string, int>();
            var document = railML2.LoadFromFile(txtFileRailML.Text);
            CountElements(document.Node, elements);
            string report = $"{txtFileRailML.Text} element count: \r\n";
            foreach (KeyValuePair<string, int> pair in elements.OrderBy(x => x.Key))
            {
                report += $"{pair.Key}";
                if (!chkEnumerateOnly.Checked)
                {
                    report += $": {pair.Value}\r\n";
                }
                else
                {
                    report += "\r\n";
                }
            }
            TextReportDlg reportDlg = new TextReportDlg();
            reportDlg.Show(report);
        }

        private void CountElements(XmlNode node, Dictionary<string, int> register)
        {
            if (node.Attributes != null)
            {
                foreach (var attribute in node.Attributes)
                {
                    XmlAttribute xmlAttribute = (XmlAttribute)attribute;
                    string key = node.Name + "@" + xmlAttribute.Name;
                    if (!register.ContainsKey(key))
                    {
                        register[key] = 0;
                    }
                    register[key]++;
                }
            }

            string nodeKey = node.Name;
            if (!register.ContainsKey(nodeKey))
            {
                register[nodeKey] = 0;
            }

            register[nodeKey]++;

            foreach (var childNode in node.ChildNodes)
            {
                XmlNode xmlChildNode = (XmlNode)childNode;
                CountElements(xmlChildNode, register);
            }
        }
    }
}
